--[[
************************************************************************

questlog.lua

File date: 2019-11-05T23:20:12Z
File revision: 
Project revision: 257
Project version: r257


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

function beql:initQuestlog()
    beql:saveLayout(QuestLogFrame);
    beql:saveLayout(QuestLogFrameCloseButton);
    beql:saveLayout(QuestFrameExitButton);
    beql:saveLayout(QuestFramePushQuestButton);
    beql:saveLayout(QuestLogListScrollFrame);
    beql:saveLayout(QuestLogDetailScrollFrame);
    beql:saveLayout(QuestLogCountRight);

    QuestLogListScrollFrame.buttons = {};
    QuestLogFrame.textures = {};
    QuestLogFrame.QUESTS_DISPLAYED = QUESTS_DISPLAYED;

    for _, child in ipairs({QuestLogFrame:GetChildren()}) do
        if(child:GetName() and child:GetName():match('QuestLogTitle')) then
            beql:initQuestLogTitleButton(child);
        end
    end

    for _, region in ipairs({QuestLogFrame:GetRegions()}) do
        if region:IsObjectType("Texture") then
            local name = region:GetTexture():match('(Icon)$') or region:GetTexture():match('([^\-]*t)$');

            if (name) then
                QuestLogFrame.textures[lcfirst(name)] = region;
            end
        end
    end

    QuestLogFrame.UIPanelWindows = {
        area = UIPanelWindows.QuestLogFrame.area,
        width = UIPanelWindows.QuestLogFrame.width,
        height = UIPanelWindows.QuestLogFrame.height
    };
end

function beql:initBeqlQuestlog()
    for i = 1, 25 do
        local frame = _G['QuestLogTitle' .. i];
        if (frame == nil) then
            frame = CreateFrame('Button', 'QuestLogTitle' .. i, QuestLogFrame, 'QuestLogTitleButtonTemplate');
            beql:initQuestLogTitleButton(frame);
            frame:ClearAllPoints();
            frame:SetPoint('TOPLEFT', _G['QuestLogTitle' .. (i - 1)], 'BOTTOMLEFT');
            frame:SetID(i);
            frame:Hide();
        end
    end

    local t = {
        {'Interface\\AddOns\\beql\\Images\\LeftOn.tga', 0},
        {'Interface\\AddOns\\beql\\Images\\RightOn.tga', 512},
    };

    for _, tInfo in ipairs(t) do
        local texture = QuestLogFrame:CreateTexture(nil, 'BORDER');
        texture:SetTexture(tInfo[1]);
        texture:SetParent(QuestLogFrame);
        texture:SetHeight(512)
        texture:SetWidth(512)
        texture:ClearAllPoints();
        texture:SetPoint('TOPLEFT', QuestLogFrame, 'TOPLEFT', tInfo[2], 0);
        texture:Hide();

        QuestLogFrame.textures['beql' .. tInfo[1]:match('(%w*)On.tga$')] = texture;
    end

    -- QuestLogDetailScrollFrame:Show is triggered by QuestLogUpdate, so we must place a hoot to keep it hidden
    hooksecurefunc(QuestLogDetailScrollFrame, 'Show', function(self) 
        if(beql:IsEnabled() and beql.db.profile.QuestLog.style == 'extended' and beql.db.char.QuestLogMinimized) then 
            self:Hide();
        end 
    end);
end

function beql:restoreBlizzardQuestlog()
    for i = QuestLogFrame.QUESTS_DISPLAYED + 1, QUESTS_DISPLAYED do
        local frame = _G['QuestLogTitle' .. i];
        if (frame ~= nil) then
            frame:Hide();
        end
    end

    QUESTS_DISPLAYED = QuestLogFrame.QUESTS_DISPLAYED;

	-- frame Defaults
	QuestLogTitleText:SetText(QUEST_LOG)
	beql:QuestLog_SetFontSize(12)
	QuestLogFrame:SetAlpha(1)
	QuestLogFrame:SetScale(1)
	QuestLogFrame:RegisterForDrag(0)

    beql:restoreLayout(QuestLogFrame);
    beql:restoreLayout(QuestLogFrameCloseButton);
    beql:restoreLayout(QuestFrameExitButton);
    beql:restoreLayout(QuestFramePushQuestButton);
    beql:restoreLayout(QuestLogListScrollFrame);
    beql:restoreLayout(QuestLogDetailScrollFrame);
    beql:restoreLayout(QuestLogCountRight);

	-- Hide Addon Buttons
	beql.frames.questLogToggleButton:Hide();
	beql.frames.configButton:Hide();

    QuestLogDetailScrollFrame:Show();

    QuestLogFrame.textures.icon:SetTexture("Interface\\QuestFrame\\UI-QuestLog-BookIcon");
	-- restore old textures
	for k, texture in pairs(QuestLogFrame.textures) do
		if k:match('beql') then
			texture:Hide()
		else
			texture:Show()
		end
	end

	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;
	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;
	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;

	-- Blizzard Status is now resored, activate beql stuff if needed
end

function beql:QuestLog_UpdateFrame()
	-- movability
	if beql.db.profile.QuestLog.locked then
		QuestLogFrame:RegisterForDrag(0);
		beql.frames.questLogAnchor:SetMovable(nil);
	else
		QuestLogFrame:RegisterForDrag("LeftButton");
		beql.frames.questLogAnchor:SetMovable(true);
	end

    QuestLogFrame:SetAlpha(beql.db.profile.QuestLog.Alpha);
	QuestLogFrame:SetScale(beql.db.profile.QuestLog.Scale);
	QuestLogFrame:ClearAllPoints();
	QuestLogFrame:SetPoint("TOPLEFT", beql.frames.questLogAnchor, "TOPLEFT", 0,0)

	return true
end

function beql:QuestLog_ExtendedQuestLog()
    QUESTS_DISPLAYED = 25;

    -- adjust button's position
    QuestLogFrameCloseButton:ClearAllPoints();
    QuestLogFrameCloseButton:SetPoint('TOPRIGHT', QuestLogFrame, 'TOPRIGHT', 2, -8);

    QuestFrameExitButton:ClearAllPoints();
    QuestFrameExitButton:SetPoint('BOTTOMRIGHT', QuestLogFrame, 'BOTTOMRIGHT', -10, 55);

    QuestFramePushQuestButton:ClearAllPoints();
    QuestFramePushQuestButton:SetPoint('TOPLEFT', QuestLogFrameAbandonButton, 'TOPRIGHT', 2);

    QuestLogCountRight:ClearAllPoints();
    QuestLogCountRight:SetPoint('TOPRIGHT', QuestLogFrame, 'TOPRIGHT', -15, -41);

    QuestLogFrame:SetAttribute("UIPanelLayout-height", 562)
    QuestLogFrame:SetHeight(562)

	-- Code from DoubleWide by Iriel
	-- {
	QuestLogFrame:SetAttribute("UIPanelLayout-width", 680);
	QuestLogFrame:SetAttribute("UIPanelLayout-height", 562);
	QuestLogFrame:SetAttribute("UIPanelLayout-area", 'doublewide');

	QuestLogListScrollFrame:SetHeight(410)

	for key, texture in pairs(QuestLogFrame.textures) do
        if key:match('beql') or key:match('icon') then
            texture:Show()
        else
            texture:Hide()
        end  
	end

    QuestLogFrame.textures.icon:SetTexture("Interface\\Addons\\beql\\Images\\icon");

	-- }
	-- Code from DoubleWide by Iriel, mod by angelike @ wowace.com

end

function beql:QuestLog_ToggleStyle(style)
    beql:restoreBlizzardQuestlog()

	if (style == 'simple' or style == 'extended') then
		beql.frames.configButton:Show();

		QuestLogFrame:ClearAllPoints();
		QuestLogFrame:SetPoint("TOPLEFT", beql.frames.questLogAnchor, "TOPLEFT", 0,0)
		QuestLogTitleText:SetText(L["Bayi's Extended Quest Log"])

		if style == 'extended' then
            beql.frames.questLogToggleButton:Show();
            beql:QuestLog_ExtendedQuestLog();

			if(beql.db.char.QuestLogMinimized or GetNumQuestLogEntries() == 0) then
                beql:QuestLog_Minimize();
            else
                beql:QuestLog_Maximize();
            end
		end

		beql:QuestLog_SetFontSize(beql.db.profile.QuestLog.FontSize);
	end

	QuestLog_Update();
end

function beql:QuestLog_Minimize(button)
	beql.db.char.QuestLogMinimized = true
	button = button or beql.frames.questLogToggleButton
	button:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
	button:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")

	QuestLogFrame:SetAttribute("UIPanelLayout-width", 384)
	QuestLogFrame:SetWidth(353)

	-- change textures
	QuestLogFrame.textures.beqlRight:Hide();
	QuestLogFrame.textures.beqlLeft:SetTexture("Interface\\AddOns\\beql\\Images\\LeftOff");

    QuestLogDetailScrollFrame:ClearAllPoints();
	QuestLogDetailScrollFrame:SetPoint("TOPLEFT", QuestLogListScrollFrame, "TOPRIGHT", 30, 2);
	QuestLogDetailScrollFrame:SetHeight(410);
	QuestLogDetailScrollFrame:Hide();
end

function beql:QuestLog_Maximize(button)
	beql.db.char.QuestLogMinimized = false

	if ( GetNumQuestLogEntries() == 0 ) then
		-- no quests in log, do not allow to maximize
		return
	end

	button = button and button or beql.frames.questLogToggleButton
	button:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	button:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")
	QuestLogFrame:SetAttribute("UIPanelLayout-width", 680)
	QuestLogFrame:SetWidth(682)


	-- change textures
	QuestLogFrame.textures.beqlRight:Show();
	QuestLogFrame.textures.beqlLeft:SetTexture("Interface\\AddOns\\beql\\Images\\LeftOn");

    QuestLogDetailScrollFrame:ClearAllPoints();
	QuestLogDetailScrollFrame:SetPoint("TOPLEFT", QuestLogListScrollFrame, "TOPRIGHT", 30, 2);
	QuestLogDetailScrollFrame:SetHeight(410);
    QuestLogDetailScrollFrame:Show();
end

function beql:initQuestLogTitleButton(button)
    tinsert(QuestLogListScrollFrame.buttons, button);

    for _, region in ipairs( {button:GetRegions()}) do
        if(region:GetName()) then
            local name = region:GetName():gsub(button:GetName(), '');
            local key = name:gsub('^.', strlower(name:match('^.'))); -- lower first char
            button[key] = region;
        end
    end

    -- save handler
    button._OnClick = button:GetScript('OnClick');

    button:SetScript('OnClick', beql.TileButtonOnClick)
end

function beql:QuestLog_SetFontSize(size)
	if not size or size == 0 then size = 12 end
	local font, fontsize, fontflags, button, buttontag
    for _, button in ipairs(QuestLogListScrollFrame.buttons) do
        button.tag:SetFont(button.tag:GetFont(), size)
        button.normalText:SetFont(button.normalText:GetFont(), size)
	end

	-- +0px
	local smallfonts = {
		'QuestInfoRequiredMoneyText','QuestInfoTimerText','QuestInfoObjective1',
		'QuestInfoObjective2','QuestInfoObjective3','QuestInfoObjective4',
		'QuestInfoObjective5','QuestInfoObjective6','QuestInfoObjective7',
		'QuestInfoObjective8','QuestInfoObjective9','QuestInfoObjective10',
	}
	
	-- +1px
	local normalfonts = {
		"QuestInfoSpellLearnText", "QuestInfoItemReceiveText", "QuestInfoItemChooseText", 
		"QuestInfoDescriptionText", "QuestInfoAnchor", "QuestInfoGroupSize", 
		"QuestInfoRewardText", "QuestInfoObjectivesText",
		
		-- test
		'QuestInfoHonorFrameReceiveText', 'QuestInfoArenaPointsFrameReceiveText', 'QuestInfoTalentFrameReceiveText', 
		'QuestInfoXPFrameReceiveText', 'QuestInfoPlayerTitleFrameTitle', 'QuestInfoPlayerTitleFrameTitleLabel', 'QuestInfoReputationText', 
	}
	
	local largefonts = {
		'QuestInfoXPFramePoints', 'QuestInfoTalentFramePoints', 
	}
	
	-- +6px
	local bigfonts = {
		"QuestInfoTitleHeader", "QuestInfoDescriptionHeader", "QuestInfoObjectivesHeader", "QuestInfoRewardsHeader" ,
	}

	for _, fontname in ipairs(smallfonts) do
		if not _G[fontname] then
			beql:debugprint(fontname.." not found")
			break
		end
		local font,fontsize,fontflags = _G[fontname]:GetFont();
		_G[fontname]:SetFont(font,size,fontflags);
	end

	for _, fontname in ipairs(normalfonts) do
		if not _G[fontname] then
			beql:debugprint(fontname.." not found")
			break
		end
		local font,fontsize,fontflags = _G[fontname]:GetFont();
		_G[fontname]:SetFont(font,size + 1,fontflags);
	end

	for _, fontname in ipairs(largefonts) do
		if not _G[fontname] then
			beql:debugprint(fontname.." not found")
			break
		end
		local font,fontsize,fontflags = _G[fontname]:GetFont();
		_G[fontname]:SetFont(font,size + 4,fontflags);
	end

	for _, fontname in ipairs(bigfonts) do
		if not _G[fontname] then
			beql:debugprint(fontname.." not found")
			break
		end
		local font,fontsize,fontflags = _G[fontname]:GetFont();
		_G[fontname]:SetFont(font,size + 6,fontflags);
	end


	--font,fontsize,fontflags = QuestLogDummyText:GetFont()
	--QuestLogDummyText:SetFont(font,size + 6)
end

--
-- Hooks
--

function beql:TileButtonOnClick(...)
    local button = self;

    if(IsShiftKeyDown()) then
        beql:toggleQuestWatch(self:GetID() + FauxScrollFrame_GetOffset(QuestLogListScrollFrame));

        return;
    end

    if (beql.db.profile.QuestLog.style == 'extended' and beql.db.char.QuestLogMinimized) then 
        beql.db.char.QuestLogMinimized = false;
        beql:QuestLog_ToggleStyle(beql.db.profile.QuestLog.style);
    end 

    if (not IsShiftKeyDown() ) then
        securecall(self._OnClick, self, ...);
    end
end

function beql:GetTaggedQuestLogTitle(index)
	local  questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(i);

	if isHeader then
		-- no need to modify heades, exit
		return questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID, beql.quests[questID].header;
	end
	
	-- build a tagged title
	if beql.db.profile.QuestLog.showlevel and questLogTitleText and level and level > 0 and not isHeader then
		if  questTag == GROUP then
			if suggestedGroup > 0 then
				questLogTitleText = "["..  level .. "G" .. suggestedGroup .. "]" .. questLogTitleText
			else
				questLogTitleText = "["..  level .. "G] " .. questLogTitleText
			end
		elseif  questTag == ELITE  then
			questLogTitleText = "[" .. level .. "+] " .. questLogTitleText
		elseif  questTag == ITEM_HEROIC then
			questLogTitleText = "[" .. level .. "H] " .. questLogTitleText
		elseif  questTag == RAID  then
			questLogTitleText = "[" .. level .. "R] " .. questLogTitleText
		elseif  questTag == PVP  then
			questLogTitleText = "[" .. level .. "P] " .. questLogTitleText
		elseif isDaily then
			questLogTitleText = "[" .. level .. "Y] " .. questLogTitleText
		elseif  questTag == LFG_TYPE_DUNGEON  then
			questLogTitleText = "[" .. level .. "D] " .. questLogTitleText
		else
			questLogTitleText = "[" .. level .. "] " .. questLogTitleText
		end
	end

	return questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, uid
end

function beql:restoreLayout(region)
    if (not region or not region.GetNumPoints) then
        return;
    end 

    region:ClearAllPoints();

    for _, point in ipairs(region._points) do
        region:SetPoint(unpack(point));
    end

    region:SetWidth(region._width);
    region:SetHeight(region._height);
end

function beql:saveLayout(region)
    if (not region or not region.GetNumPoints) then
        return;
    end 

    region._points = {};

    for i = 1, region:GetNumPoints() do
        tinsert(region._points, {region:GetPoint(i)});
    end

    region._width = region:GetWidth();
    region._height = region:GetHeight();
end


--- EOF --- 
 
